<?php

namespace DTInternet\DtiOnlineconsulting\Controller;

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2022 Stefan Bublies <bublies@dt-internet.de>
 *
 *  All rights reserved
 *
 ***************************************************************/

use \TYPO3\CMS\Core\Utility\GeneralUtility;
use \TYPO3\CMS\Extbase\Mvc\Controller\ActionController;
use DTInternet\DtiUsermanagement\Utility\FeuserUtility;
use TYPO3\CMS\Extbase\Utility\LocalizationUtility as Locallang;
use \Datetime;
use DTInternet\DtiOnlineconsulting\Utility\MailUtility;

/**
 * (c) 2022 Stefan Bublies bublies@dt-internet.de
 */
class VideomeetController extends ActionController
{

    /**
     * @var \TYPO3\CMS\Extbase\Persistence\Generic\PersistenceManager
     * @TYPO3\CMS\Extbase\Annotation\Inject
     */
    protected $persistenceManager;

    /**
     * Init the persistenceManager
     *
     * @return void
     */
    public function __construct()
    {
        $this->persistenceManager = \TYPO3\CMS\Core\Utility\GeneralUtility::makeInstance("TYPO3\\CMS\\Extbase\\Persistence\\Generic\\PersistenceManager");
    }

    /**
     * videoconsultingRepository
     *
     * @var \DTInternet\DtiOnlineconsulting\Domain\Repository\VideoconsultingRepository
     */
    protected $videoconsultingRepository = null;

    /**
     * @param \DTInternet\DtiOnlineconsulting\Domain\Repository\VideoconsultingRepository $videoconsultingRepository
     */
    public function injectVideoconsultingRepository(
        \DTInternet\DtiOnlineconsulting\Domain\Repository\VideoconsultingRepository $videoconsultingRepository
    ) {
        $this->videoconsultingRepository = $videoconsultingRepository;
    }

    /**
     * @return void
     */
    public function videorequestAction()
    {

        $arguments = $this->request->getArguments();
        $feUserData = FeuserUtility::get();

        // Store in Database
        if ($arguments['videomeetrequest']) {
            $videoId = $this->gen_videoId();
            $password = substr($videoId, 25, -1);
            $videoIdwithCustomerID = $this->getSettings()['identity'].'-' . $videoId;
            if ($feUserData['usergroup'] > $this->getSettings()['agentGroupUid']) {
                $newRequest = new \DTInternet\DtiOnlineconsulting\Domain\Model\Videoconsulting();
                $newRequest->setVideoid($videoIdwithCustomerID);
                $newRequest->setPid($this->getSettings()['storagePid']);
                $newRequest->setFeuser($feUserData['uid']);
                $newRequest->setAgent(0);
                $newRequest->setInformation($arguments['videomeetrequest']['information']);
                $newRequest->setPassword($password);
                $newRequest->setConfirmed(0);
            }

            $this->videoconsultingRepository->add($newRequest);
            $this->persistenceManager->persistAll();

            $this->addFlashMessage(
                Locallang::translate('tx_dtionlineconsulting.videomeet.success.body', 'dti_onlineconsulting', null),
                Locallang::translate('tx_dtionlineconsulting.videomeet.success.header', 'dti_onlineconsulting', null),
                \TYPO3\CMS\Core\Messaging\AbstractMessage::OK
            );
            $this->view->assignMultiple(['password' => $password]);
        } else {
            $this->view->assignMultiple([
                'feuser' => $feUserData,
                'newRequest' => 1,
                'openVideoDates' => $this->videoconsultingRepository->findByFeuser($feUserData['uid'])
            ]);
        }
    }

    /**
     * @param \DTInternet\DtiOnlineconsulting\Domain\Model\Videoconsulting $videoconsulting
     * @return void
     */
    public function showAction(\DTInternet\DtiOnlineconsulting\Domain\Model\Videoconsulting $videoconsulting)
    {
        $arguments = $this->request->getArguments();

        if ($arguments['meetingvalidate']) {
            if ($arguments['meetingvalidate']['password'] == $videoconsulting->getPassword()) {
                $url = 'https://call.'.$this->getSettings()['identity'].'.apps.hosting/' . $videoconsulting->getVideoid();
                $this->view->assignMultiple(['date' => $videoconsulting, 'url' => $url, 'state' => 1]);
            } else {
                $this->addFlashMessage(
                    Locallang::translate('tx_dtionlineconsulting.videomeet.password.error.body', 'dti_onlineconsulting',
                        null),
                    Locallang::translate('tx_dtionlineconsulting.videomeet.password.error.header',
                        'dti_onlineconsulting', null),
                    \TYPO3\CMS\Core\Messaging\AbstractMessage::ERROR
                );
                $this->view->assignMultiple(['state' => 2]);
            }
        } else {
            $this->view->assignMultiple(['date' => $videoconsulting]);
        }
    }

    /**
     * @return void
     */
    public function listAction()
    {
        $this->view->assignMultiple([
            'openVideoDates' => $this->videoconsultingRepository->findByConfirmed(0),
            'confirmedVideoDates' => $this->videoconsultingRepository->findByConfirmed(1)
        ]);
    }

    /**
     * @param \DTInternet\DtiOnlineconsulting\Domain\Model\Videoconsulting $videoconsulting
     * @return void
     */
    public function activateAction(\DTInternet\DtiOnlineconsulting\Domain\Model\Videoconsulting $videoconsulting)
    {
        $arguments = $this->request->getArguments();

        // Customer-Infos
        $customer = FeuserUtility::readFeUserDataSingle($videoconsulting->getFeuser());

        if ($arguments['confirmmeeting']) {
            $videoconsulting->setPreferreddate(new DateTime($arguments['confirmmeeting']['preferreddate']));
            $videoconsulting->setTimestampduration($arguments['confirmmeeting']['timestampduration']);
            $videoconsulting->setMaxuser($arguments['confirmmeeting']['maxuser']);
            $videoconsulting->setConfirmed(1);
            $this->videoconsultingRepository->update($videoconsulting);
            $this->persistenceManager->persistAll();

            // Info-Mail to Customer

            if($customer->getUsergroup()[0]->getUid() == $this->getSettings()['fullUserGroupUid']) {
                $content = [];
                $content['feuser'] = $customer;
                $content['videodate'] = new DateTime($arguments['confirmmeeting']['preferreddate']);
                //send mail
                $mail = new MailUtility();
                $mail->sendMail($customer->getEmail(), 'VideodateConfirmation', $content, 'Bestätigung der Videotermin-Anfrage');
            }

            $uriBuilder = $this->controllerContext->getUriBuilder();
            $uri = $uriBuilder
                ->reset()
                ->uriFor('list', array(), 'Videomeet', 'DtiOnlineconsulting', 'Videomeetagent');
            $this->redirectToURI($uri, 0, 301);
        } else {
            $this->view->assignMultiple(['date' => $videoconsulting, 'customer' => $customer]);
        }
    }

    /**
     * @param \DTInternet\DtiOnlineconsulting\Domain\Model\Videoconsulting $videoconsulting
     * @return void
     */
    public function meetingRedirectAction(\DTInternet\DtiOnlineconsulting\Domain\Model\Videoconsulting $videoconsulting)
    {
        // Set Startdate
        $videoconsulting->setVideoStarttime(strtotime("now"));
        $this->videoconsultingRepository->update($videoconsulting);
        $this->persistenceManager->persistAll();

        // Redirect to Video-API
        $uri = 'https://call.'.$this->getSettings()['identity'].'.apps.hosting/'.$videoconsulting->getVideoid();
        $this->redirectToURI($uri, $delay = 0, $statusCode = 303);
    }

    /**
     * @param \DTInternet\DtiOnlineconsulting\Domain\Model\Videoconsulting $videoconsulting
     * @return void
     */
    public function declineAction(\DTInternet\DtiOnlineconsulting\Domain\Model\Videoconsulting $videoconsulting)
    {
        $videoconsulting->setConfirmed(2);
        $this->videoconsultingRepository->update($videoconsulting);
        $this->persistenceManager->persistAll();

        // Info-Mail to Customer
        $customer = FeuserUtility::readFeUserDataSingle($videoconsulting->getFeuser());
        if($customer->getUsergroup()[0]->getUid() == $this->getSettings()['fullUserGroupUid']) {
            //send mail
            $mail = new MailUtility();
            $mail->sendMail($customer->getEmail(), 'VideodateDecline', array(), 'Videotermin-Anfrage abgelehnt');
        }

        $uriBuilder = $this->controllerContext->getUriBuilder();
        $uri = $uriBuilder
            ->reset()
            ->uriFor('list', array(), 'Videomeet', 'DtiOnlineconsulting', 'Videomeetagent');
        $this->redirectToURI($uri, 0, 301);
    }


    /**
     * @return string
     */
    public function gen_videoId() {
        return sprintf( '%04x%04x-%04x-%04x-%04x-%04x%04x%04x',
            // 32 bits for "time_low"
            mt_rand( 0, 0xffff ), mt_rand( 0, 0xffff ),

            // 16 bits for "time_mid"
            mt_rand( 0, 0xffff ),

            // 16 bits for "time_hi_and_version",
            // four most significant bits holds version number 4
            mt_rand( 0, 0x0fff ) | 0x4000,

            // 16 bits, 8 bits for "clk_seq_hi_res",
            // 8 bits for "clk_seq_low",
            // two most significant bits holds zero and one for variant DCE1.1
            mt_rand( 0, 0x3fff ) | 0x8000,

            // 48 bits for "node"
            mt_rand( 0, 0xffff ), mt_rand( 0, 0xffff ), mt_rand( 0, 0xffff )
        );
    }

    /**
     * Returns TypoSript settings array
     *
     * @param string $extension Name of the extension
     * @param string $plugin Name of the plugin
     * @return array
     */
    public function getSettings()
    {
        $configurationManager = GeneralUtility::makeInstance('TYPO3\\CMS\\Extbase\\Configuration\\ConfigurationManager');

        $typoScript = $configurationManager->getConfiguration(
            \TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface::CONFIGURATION_TYPE_FULL_TYPOSCRIPT);

        $settings = $typoScript['tx_dtionlineconsulting_settings.'];

        return $settings;
    }

}
