<?php
namespace DTInternet\DtiUsermanagement\Services;

/***
 * This file is part of the "dti_usermanagement" Extension for TYPO3 CMS.
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 *  (c) 2022 Stefan Bublies <bublies@dt-internet.de>, D&T Internet GmbH
 ***/

use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Core\Database\Connection;
use TYPO3\CMS\Core\Database\ConnectionPool;
use TYPO3\CMS\Core\Database\Query\QueryBuilder;

class AgentState
{

    /**
     * * (c) 2023 Stefan Bublies
     */

    public function cleanState()
    {
        $this->cleanStateDatas();
    }

    public function cleanStateDatas()
    {

        /** @var Connection $connection */
        $connection = GeneralUtility::makeInstance(ConnectionPool::class)->getConnectionForTable('fe_users');
        /** @var QueryBuilder $queryBuilder */
        $queryBuilder = $connection->createQueryBuilder();
        $feuserDatas = $queryBuilder
            ->select('*')
            ->from('fe_users')
            ->where($queryBuilder->expr()->eq('usergroup', $queryBuilder->createNamedParameter($this->getSettings()['agentGroupUid'], \PDO::PARAM_INT)))
            ->execute()
            ->fetchAll(\Doctrine\DBAL\FetchMode::ASSOCIATIVE);

        $time = strtotime("now");
        foreach($feuserDatas as $data) {
            $connection = GeneralUtility::makeInstance(ConnectionPool::class)->getConnectionForTable('fe_users');
            $connection->update(
                'fe_users',
                [
                    'tstamp' => $time,
                    'isonline' => 0,
                    'chatavailable' => 0,
                ],
                [
                    'uid' => $data['uid']
                ]
            );
        }
    }

    /**
     * Returns TypoSript settings array
     *
     * @param string $extension Name of the extension
     * @param string $plugin Name of the plugin
     * @return array
     */
    public function getSettings()
    {
        $configurationManager = GeneralUtility::makeInstance('TYPO3\\CMS\\Extbase\\Configuration\\ConfigurationManager');

        $typoScript = $configurationManager->getConfiguration(
            \TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface::CONFIGURATION_TYPE_FULL_TYPOSCRIPT);

        $settings = $typoScript['tx_dtionlineconsulting_settings.'];

        return $settings;
    }
}
